use std::{fmt::Display, str::Utf8Error};

use anyhow;
use thiserror::Error;
use tokio::time::error::Elapsed;

pub type Result<T> = std::result::Result<T, ServerError>;

#[derive(Debug, Error)]
pub enum ServerError {
    Timeout(Elapsed),
    Anyhow(anyhow::Error),
    IO(std::io::Error),
    Format(Utf8Error),
}

impl From<anyhow::Error> for ServerError {
    fn from(value: anyhow::Error) -> Self {
        Self::Anyhow(value)
    }
}

impl From<std::io::Error> for ServerError {
    fn from(value: std::io::Error) -> Self {
        Self::IO(value)
    }
}

impl From<Utf8Error> for ServerError {
    fn from(value: Utf8Error) -> Self {
        Self::Format(value)
    }
}

impl From<Elapsed> for ServerError {
    fn from(value: Elapsed) -> Self {
        Self::Timeout(value)
    }
}

impl Display for ServerError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{:?}", self)
    }
}
